;;; nndiary-to-diary.el --- Make a diary file from nndiary group

;;; Author Jody Klymak jklymak@oce.orst.edu
;;
;;  No waranty, implied or otherwise....

;; Commentary....
;;
;; Hey, Didier Verna's nndiary is a great way to keep track of your
;; schedule inside of gnus.  However, it'd be nice to see those
;; entries marked on your emacs calendar and in your fancy diary
;; display, right?  Thats what this file does.
;; 
;; This file provides a hook that should be added to
;; nndiary-request-update-info-hooks.
;;
;;(add-hook 'nndiary-request-update-info-hooks 'gnus-nndiary-update-info-hook) 
;;
;; I had to change the arguments of
;; 'nndiary-request-update-info-hooks, so please use my copy of
;; gnus-diary.el and nndiary.el until Didier gets back and can look
;; this over.
;;
;; What does the hook do?  For each diary group, it creates a diary
;; file.  The diary file gets stored in nndiary-current-directory, ie,
;; in with your messages, and is named "diary".  For my set up it gets
;; put in "~/News/diary/diary/diary".
;;
;; This diary can then be "included" in your ~/diary, i.e.
;; 
;; #include "~/News/diary/diary/diary"
;; 
;; It makes a "diary" file for each diary group, so I actually have
;;  
;; #include "~/News/diary/diary/diary"
;; #include "~/News/diary/birthdays/diary"
;;
;; In order for these files to be marked on your calendar and to show
;; up in your fancy diary displays you'll need to put this in your
;; init.el or .emacs.
;;
;; (add-hook 'list-diary-entries-hook 'include-other-diary-files)
;; (add-hook 'mark-diary-entries-hook 'mark-included-diary-files)
;;
;; You will, or course, need to (require 'nndiary-to-diary) in your .gnus.
;;
;; The diary files that are made are rebuilt from scratch each time
;; the diary group is updated, so don't edit them if you want to keep
;; information around.  They are only meant as a mirror of whats in
;; your nndiary group.
;;   
;; Any comments are most welcome - this is my first lisp project.  I'm
;; sure there were many things that I hacked together that could be
;; much more efficient.  Use at your own risk!
;; 
;;  I also tossed in gnus-diary-add-header which adds the X-DIARY
;; headers below Subject in the current buffer.  Useful to forward
;; yourself messages from other groups...
;;

(require 'nndiary)

(defun gnus-nndiary-to-diary (group info)
  "Convert the diary entries in GROUP to a diary file.  The diary file
gets put in nndiary-current-directory, and is called diary.  Meant to
be a hook to nndiary-request-update-info-hooks."
  (nndiary-possibly-change-directory group)
  (let ((articles (nndiary-flatten (gnus-info-read info) 0))
	article file unread buf
        (nndiary-file (concat nndiary-current-directory "diary")))
    ;; load the old diary file and erase it
    (save-window-excursion
      (save-restriction
	(find-file nndiary-file)
	(erase-buffer)))
    ;; loop through all the articles.  If they are not empty, then add
    ;; an entry to the diary
    (save-window-excursion
      (save-restriction
	(setq buf (nnheader-set-temp-buffer " *nndiary-diary*"))
	(while (setq article (pop articles))
	  (setq file (concat nndiary-current-directory
			     (int-to-string article)))
	  (erase-buffer)
	  (if (nnheader-insert-head file)
	      (save-window-excursion 
		(save-restriction
		  (nndiary-entry-to-diary nndiary-file)))))
	(find-file nndiary-file)
	(save-buffer)
	(kill-this-buffer)))))


(defun nndiary-entry-to-diary (nndiary-file)
 "Take the current buffer and add an diary entry to the file in
NNDIARY-FILE"
  (interactive) 
 ;; nndiary-schedule returns a list (ummm, or is it an a-list) that 
  ;; has the following format: 
  ;;((m) (h) (d) (m) (y) (d) (t))
  ;; where the value is a list of integers, or nil (or a time zone).
  (let* 
     (
      (sched (nndiary-schedule))
      (minute-list (if (nth 0 sched)
		       (sort (nndiary-flatten (nth 0 sched) 0 59) '<)
		     nil))
      (hour-list (if (nth 1 sched) 
		     (sort (nndiary-flatten (nth 1 sched) 0 23) '<)
		   nil))
      (dom-list (if (nth 2 sched)
		 (sort (nndiary-flatten (nth 2 sched) 1 31) '<)
	       nil))
      (month-list (if (nth 3 sched)
		      (sort (nndiary-flatten (nth 3 sched) 1 12) '<)
                    nil))
      (year-list (if (nth 4 sched)
		 (sort (nndiary-flatten (nth 4 sched) 1971) '<)
	       nil))
      (dow-list (if (nth 5 sched)
		 (sort (nndiary-flatten (nth 5 sched) 0 6) '<)
	       nil))
      )
    ;; get the subject
    (let* ((subhead "^Subject: \\(.*\\)$"))
       (goto-char (point-min))
       (re-search-forward subhead nil t)
       (setq subject (match-string 1)))
    ;; get the XRef  
    (let* ((subhead "^Xref: \\(.*\\)$"))
       (goto-char (point-min))
       (re-search-forward subhead nil t)
       (setq xref (match-string 1)))
    (setq subject (format "%s   [nndiary-Xref: %s]"  subject xref))
    ;;How many forms are there?
    ;; 1) Only hhmm are specified - "hhmm Subject"
    ;; 2) one of m/d/y are specified - "m/d/y (hhmm) Subject"
    ;; 3) a day is specified - "D (hhmm) Subject"
    ;;
    ;; make hhmm strings...
    (setq hhmm nil)
    (setq outputstrs nil)
    (if hour-list
	(while hour-list
	  (setq hour (car hour-list))
	  (if minute-list
	      (while minute-list
		(setq minute (car minute-list))
		(setq minute-list (cdr minute-list))
		(setq hhmm (cons (format "%02d%02d" hour minute) hhmm)))
	    (setq hhmm (cons (format "%02d%02d" hour 0) hhmm)))
	  (setq hour-list (cdr hour-list)))
      (setq hhmm (list "")))
    ;; take care of case where no DOW and no year/month/day...
    (if (or dom-list month-list year-list dow-list)
     ()
     (while hhmm
       (setq hhmmm (car hhmm))
       (setq hhmm (cdr hhmm))
       (setq outputstrs (cons (format "%s %s" hhmmm subject) outputstrs))))

    ;; Take care of case where there is a DOW field....
    (setq hhmmorig hhmm)
    (setq daystrs '("Sun" "Mon" "Tue" "Wed" "Thu" "Fri" "Sat"))
    (if dow-list 
	(while dow-list
          (setq hhmm hhmmorig)
	  (setq dow (car dow-list))
	  (setq dowstr (format (nth dow daystrs)))
          (while hhmm
	    (setq hhmmm (car hhmm))
	    (setq hhmm (cdr hhmm))
	    (setq outputstrs (cons (format "%s %s %s" dowstr hhmmm subject) outputstrs)))
	  (setq dow-list (cdr dow-list)))
       ; else
      )
    (setq hhmm hhmmorig)
    ;; Take care of other cases (note not exclusive with DOW 
    (if (null (or dom-list month-list year-list))
        ()
        ;; get the monthstrs...
        (setq monthstrs nil)
	(if (null month-list)
           (setq monthstrs (list "*"))
           (while month-list
              (setq month (car month-list))
	      (setq month-list (cdr month-list))
              (setq monthstrs (cons (format "%s" month) monthstrs))))
        ;; get the domstrs...
        (setq domstrs nil)
	(if (null dom-list)
           (setq domstrs (list "*"))
           (while dom-list
              (setq dom (car dom-list))
	      (setq dom-list (cdr dom-list))
              (setq domstrs (cons (format "%s" dom) domstrs))))
        ;; get the yearstrs...
        (setq yearstrs nil)
	(if (null year-list)
           (setq yearstrs (list "*"))
           (while year-list
              (setq year (car year-list))
	      (setq year-list (cdr year-list))
              (setq yearstrs (cons (format "%s" year) yearstrs))))
        ;; Now we have the four sets of strings, all which must be cobbled
        ;; together 
        (setq domstro domstrs)
        (setq yearstro yearstrs)
        (setq hhmmo hhmm)
        (while monthstrs
	  (setq monthstr (car monthstrs))	  
	  (setq monthstrs (cdr monthstrs))
          (setq domstrs domstro)
	  (while domstrs
	    (setq domstr (car domstrs))	  
	    (setq domstrs (cdr domstrs))
	    (setq yearstrs yearstro)
	    (while yearstrs
	      (setq yearstr (car yearstrs))	  
	      (setq yearstrs (cdr yearstrs))
	      (setq hhmm hhmmo)
	      (while hhmm
		(setq hhmmm (car hhmm))	  
		(setq hhmm (cdr hhmm))
		(setq outputstrs (cons (format "%s/%s/%s %s %s" monthstr domstr yearstr hhmmm subject) outputstrs)))))))
  (setq output outputstrs)
  (while outputstrs 
    (make-diary-entry (car outputstrs) nil nndiary-file )
    (setq outputstrs (cdr outputstrs)))
))


(defun gnus-diary-add-header ()
  "Add nndiary compatible headers to the current buffer"   
  (interactive "*")
; Add an nndiary compatible header to a message
  (save-excursion
    (save-restriction
      (goto-char (point-min))
      (cond ((re-search-forward "^Subject: " nil t)
	     (goto-line (+ 1 (line-number)))
	     (let* ((heads nndiary-headers))
	       (while heads
		 (setq head (car heads))
		 (setq heads (cdr heads))
		 (insert (format "X-Diary-%s: *\n" (car head))))))
             (t (message "Not a mail message")))
      )))

(provide 'nndiary-to-diary)
